using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using BinaryIntellect.DataAccess;

public class MyMembershipProvider:MembershipProvider
{

    private DatabaseHelper db = null;

    #region General

    public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
    {
        string connstr = ConfigurationManager.ConnectionStrings[config["connectionStringName"]].ConnectionString;
        db = new DatabaseHelper(connstr);
    }

    public override string Name
    {
        get
        {
            return "MyMembershipProvider";
        }
    }
    #endregion

    #region User Management

    public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
    {
        MembershipUser user = new MembershipUser(Name, username, providerUserKey, email, passwordQuestion, null, isApproved, false, DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now);
        string sql = "INSERT INTO USERS(USERNAME,PASSWORD,EMAIL,ISACTIVE) VALUES(@UID,@PWD,@EMAIL,@ISACTIVE)";
        db.AddParameter("@UID", username);
        db.AddParameter("@PWD", password);
        db.AddParameter("@EMAIL", email);
        db.AddParameter("@ISACTIVE", (isApproved == true ? "Y" : "N"));
        int i = db.ExecuteNonQuery(sql);

        if (i > 0)
        {
            status = MembershipCreateStatus.Success;
            return user;
        }
        else
        {
            status = MembershipCreateStatus.ProviderError;
            return null;
        }

    }

    public override void UpdateUser(MembershipUser user)
    {
        string sql = "UPDATE USERS SET EMAIL=@EMAIL,ISACTIVE=@ISACTIVE WHERE USERNAME=@UID";
        db.AddParameter("@EMAIL", user.Email);
        db.AddParameter("@ISACTIVE", (user.IsApproved ? "Y" : "N"));
        db.AddParameter("@UID", user.UserName);
        int i = db.ExecuteNonQuery(sql);
    }

    public override bool DeleteUser(string username, bool deleteAllRelatedData)
    {
        string sql = "DELETE FROM USERS WHERE USERNAME=@UID";
        db.AddParameter("@UID", username);
        int i = db.ExecuteNonQuery(sql);
        if (i > 0)
            return true;
        else
            return false;

    }

    public override MembershipUser GetUser(string username, bool userIsOnline)
    {
        MembershipUser user = null;
        string sql = "SELECT * FROM USERS WHERE USERNAME=@UID AND ISACTIVE='Y'";
        db.AddParameter("@UID", username);
        SqlDataReader reader = (SqlDataReader)db.ExecuteReader(sql);
        while (reader.Read())
        {
            user = new MembershipUser(Name, reader.GetString(reader.GetOrdinal("username")), null, reader.GetString(reader.GetOrdinal("email")), null, null, (reader.GetString(reader.GetOrdinal("isactive")) == "Y" ? true : false), false, DateTime.MinValue, DateTime.MinValue, DateTime.MinValue, DateTime.MinValue, DateTime.MinValue);
        }
        reader.Close();
        return user;
    }

    public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
    {
        MembershipUserCollection users = new MembershipUserCollection();
        object obj=db.ExecuteScalar("SELECT COUNT(*) FROM USERS");
        int reccount=0;
        if (obj != null)
            reccount = (int)obj;
        SqlDataReader reader = (SqlDataReader)db.ExecuteReader("SELECT * FROM USERS ORDER BY USERNAME");
        while (reader.Read())
        {
            MembershipUser user = new MembershipUser(Name, reader.GetString(reader.GetOrdinal("username")), null, reader.GetString(reader.GetOrdinal("email")), null, null, (reader.GetString(reader.GetOrdinal("isactive")) == "Y" ? true : false), false, DateTime.MinValue, DateTime.MinValue, DateTime.MinValue, DateTime.MinValue, DateTime.MinValue);
            users.Add(user);
        }
        reader.Close();
        totalRecords = reccount;
        return users;
    }

    #endregion

    #region Password Management

    public override bool ChangePassword(string username, string oldPassword, string newPassword)
    {
        string sql = "UPDATE USERS SET PASSWORD=@NEWPWD WHERE USERNAME=@UID AND PASSWORD=@OLDPWD";
        db.AddParameter("@NEWPWD", newPassword);
        db.AddParameter("@UID", username);
        db.AddParameter("@OLDPWD", oldPassword);
        int i = db.ExecuteNonQuery(sql);
        if (i > 0)
            return true;
        else
            return false;
    }

    public override string GetPassword(string username, string answer)
    {
        string sql = "SELECT PASSWORD FROM USERS WHERE USERNAME=@UID";
        db.AddParameter("@UID", username);
        object obj = db.ExecuteScalar(sql);
        if (obj != null)
            return obj.ToString();
        else
            return "";
    }
    #endregion

    #region Authentication

    public override bool ValidateUser(string username, string password)
    {
        string sql = "SELECT USERNAME FROM USERS WHERE USERNAME=@UID AND PASSWORD=@PWD";
        db.AddParameter("@PWD", password);
        db.AddParameter("@UID", username);
        string uid = db.ExecuteScalar(sql) as string;
        if (uid == null)
        {
            return false;
        }
        else
        {
            return true;
        }
    }
    #endregion

    #region Provider Configuration

    public override bool EnablePasswordReset
    {
        get
        {
            return false;
        }
    }

    public override bool EnablePasswordRetrieval
    {
        get 
        {
            return true;
        }
    }

    public override bool RequiresQuestionAndAnswer
    {
        get
        {
            return false;
        }
    }

    public override bool RequiresUniqueEmail
    {
        get
        {
            return false;
        }
    }
    #endregion

    #region Members Not Implemented

    public override string ApplicationName
    {
        get
        {
            throw new Exception("The method or operation is not implemented.");
        }
        set
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }

    public override int MaxInvalidPasswordAttempts
    {
        get
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }

    public override int MinRequiredNonAlphanumericCharacters
    {
        get
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }

    public override int MinRequiredPasswordLength
    {
        get
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }

    public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer)
    {
        throw new Exception("The method or operation is not implemented.");
    }

    public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
    {
        throw new Exception("The method or operation is not implemented.");
    }

    public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
    {
        throw new Exception("The method or operation is not implemented.");
    }

    public override int GetNumberOfUsersOnline()
    {
        throw new Exception("The method or operation is not implemented.");
    }

    public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
    {
        throw new Exception("The method or operation is not implemented.");
    }

    public override string GetUserNameByEmail(string email)
    {
        throw new Exception("The method or operation is not implemented.");
    }
    
    public override int PasswordAttemptWindow
    {
        get { throw new Exception("The method or operation is not implemented."); }
    }

    public override MembershipPasswordFormat PasswordFormat
    {
        get { throw new Exception("The method or operation is not implemented."); }
    }

    public override string PasswordStrengthRegularExpression
    {
        get { throw new Exception("The method or operation is not implemented."); }
    }

    public override string ResetPassword(string username, string answer)
    {
        throw new Exception("The method or operation is not implemented.");
    }

    public override bool UnlockUser(string userName)
    {
        throw new Exception("The method or operation is not implemented.");
    }

    #endregion

}
