using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using BinaryIntellect.DataAccess;
using System.Collections;

public class MyRolesProvider:RoleProvider
{
    private DatabaseHelper db = null;

    #region General

    public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
    {
        string connstr = ConfigurationManager.ConnectionStrings[config["connectionStringName"]].ConnectionString;
        db = new DatabaseHelper(connstr);
    }

    public override string Name
    {
        get
        {
            return "MyRolesProvider";
        }
    }
    #endregion

    #region Role Management

    public override void CreateRole(string roleName)
    {
        db.AddParameter("@ROLE", roleName);
        db.ExecuteNonQuery("INSERT INTO ROLES(ROLENAME) VALUES(@ROLE)");
    }

    public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
    {
        db.AddParameter("@ROLE", roleName);
        int i = db.ExecuteNonQuery("DELETE FROM ROLES WHERE ROLENAME=@ROLE");
        if (i > 0)
            return true;
        else
            return false;
    }

    public override string[] GetAllRoles()
    {
        int reccount = (int)db.ExecuteScalar("SELECT COUNT(*) FROM ROLES");
        SqlDataReader reader = (SqlDataReader)db.ExecuteReader("SELECT ROLENAME FROM ROLES");
        string[] roles = new string[reccount];
        int i = 0;
        while (reader.Read())
        {
            roles[i] = reader.GetString(0);
            i++;
        }
        reader.Close();
        return roles;
    }

    public override bool RoleExists(string roleName)
    {
        db.AddParameter("@RID", roleName);
        object obj = db.ExecuteScalar("SELECT ROLENAME FROM ROLES WHERE ROLENAME=@RID");
        if (obj != null)
        {
            return true;
        }
        else
        {
            return false;
        }

    }

    #endregion

    #region Users and Roles

    public override void AddUsersToRoles(string[] usernames, string[] roleNames)
    {
        foreach (string user in usernames)
        {
            foreach (string role in roleNames)
            {
                db.AddParameter("@UID", user);
                db.AddParameter("@RID", role);
                db.ExecuteNonQuery("INSERT INTO USERROLES(USERNAME,ROLENAME) VALUES(@UID,@RID)");
            }
        }
    }

    public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
    {
        foreach (string user in usernames)
        {
            foreach (string role in roleNames)
            {
                db.AddParameter("@UID", user);
                db.AddParameter("@RID", role);
                db.ExecuteNonQuery("DELETE FROM USERROLES WHERE USERNAME=@UID AND ROLENAME=@RID");
            }
        }
    }

    public override string[] GetRolesForUser(string username)
    {
        db.AddParameter("@UID", username);
        int reccount = (int)db.ExecuteScalar("SELECT COUNT(*) FROM USERROLES WHERE USERNAME=@UID");
        string[] roles = new string[reccount];

        db.AddParameter("@UID", username);
        SqlDataReader reader = (SqlDataReader)db.ExecuteReader("SELECT ROLENAME FROM USERROLES WHERE USERNAME=@UID");
        int i = 0;
        while (reader.Read())
        {
            roles[i] = reader.GetString(0);
            i++;
        }
        reader.Close();
        return roles;
    }

    public override string[] GetUsersInRole(string roleName)
    {
        db.AddParameter("@RID", roleName);
        int reccount = (int)db.ExecuteScalar("SELECT COUNT(*) FROM USERROLES WHERE ROLENAME=@RID");
        string[] users = new string[reccount];

        db.AddParameter("@RID", roleName);
        SqlDataReader reader = (SqlDataReader)db.ExecuteReader("SELECT USERNAME FROM USERROLES WHERE ROLENAME=@RID");
        int i = 0;
        while (reader.Read())
        {
            users[i] = reader.GetString(0);
            i++;
        }
        reader.Close();
        return users;

    }

    public override bool IsUserInRole(string username, string roleName)
    {
        db.AddParameter("@UID", username);
        db.AddParameter("@RID", roleName);
        object obj = db.ExecuteScalar("SELECT ROLENAME FROM USERROLES WHERE USERNAME=@UID AND ROLENAME=@RID");
        if (obj != null)
        {
            return true;
        }
        else
        {
            return false;
        }
    }


    #endregion

    #region Members Not Implemented

    public override string ApplicationName
    {
        get
        {
            throw new Exception("The method or operation is not implemented.");
        }
        set
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }

    public override string[] FindUsersInRole(string roleName, string usernameToMatch)
    {
        throw new Exception("The method or operation is not implemented.");
    }

    #endregion

}
